/*
Copyright (C) 2006 The Pentagram team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "pent_include.h"

#include "RemorseGame.h"

#include "SettingManager.h"
#include "FileSystem.h"
#include "IDataSource.h"
#include "PaletteManager.h"
#include "ObjectManager.h"
#include "World.h"
#include "XFormBlend.h"
#include "GameData.h"
#include "GUIApp.h"
#include "RawArchive.h"
#include "ItemFactory.h"
#include "MainActor.h"

RemorseGame::RemorseGame() : Game()
{
	// Set some defaults for gameplay-related settings
	SettingManager* settingman = SettingManager::get_instance();
	settingman->setDefault("skipstart", false);
	settingman->setDefault("endgame", false);
	settingman->setDefault("footsteps", true);
	settingman->setDefault("textdelay", 5);
}

RemorseGame::~RemorseGame()
{

}

bool RemorseGame::loadFiles()
{
	// Load palette
	pout << "Load Palette" << std::endl;
	IDataSource *pf = FileSystem::get_instance()->ReadFile("@game/static/gamepal.pal");
	if (!pf) {
		perr << "Unable to load static/gamepal.pal." << std::endl;
		return false;
	}

	IBufferDataSource xfds(U8XFormPal,1024);
	PaletteManager::get_instance()->load(PaletteManager::Pal_Game, *pf, xfds);
	delete pf;

	pout << "Load GameData" << std::endl;
	GameData::get_instance()->loadRemorseData();

	return true;
}

bool RemorseGame::startGame()
{
	// NOTE: assumes the entire engine has been reset!

	pout << "Starting new Crusader: No Remorse game." << std::endl;

	ObjectManager* objman = ObjectManager::get_instance();

	// reserve a number of objids just in case we'll need them sometime
	for (uint16 i = 384; i < 512; ++i)
		objman->reserveObjId(i);

	// FIXME: fix flags and such
	Actor* actor = ItemFactory::createActor(1,0,0,Item::FLG_IN_NPC_LIST,
											1,1,Item::EXT_PERMANENT_NPC,false);
	if (!actor) {
		perr << "Couldn't create MainActor. Exiting." << std::endl;
		exit(-1);
	}

	ObjectManager::get_instance()->assignActorObjId(actor, 1);

	actor->setLocation(60700, 59420, 16); 


	World::get_instance()->switchMap(1);

	GUIApp::get_instance()->setAvatarInStasis(true);

	return true;
}

bool RemorseGame::startInitialUsecode(const std::string &)
{
//	Process* proc = new StartU8Process();
//	Kernel::get_instance()->addProcess(proc);

	return true;
}


ProcId RemorseGame::playIntroMovie()
{
	return 0;
}

ProcId RemorseGame::playEndgameMovie()
{
	return 0;
}

void RemorseGame::playCredits()
{

}

void RemorseGame::writeSaveInfo(ODataSource* ods)
{
#if 0
	MainActor* av = getMainActor();
	sint32 x,y,z;

	std::string avname = av->getName();
	uint8 namelength = static_cast<uint8>(avname.size());
	ods->write1(namelength);
	for (unsigned int i = 0; i < namelength; ++i)
		ods->write1(static_cast<uint8>(avname[i]));

	av->getLocation(x,y,z);
	ods->write2(av->getMapNum());
	ods->write4(static_cast<uint32>(x));
	ods->write4(static_cast<uint32>(y));
	ods->write4(static_cast<uint32>(z));

	ods->write2(av->getStr());
	ods->write2(av->getInt());
	ods->write2(av->getDex());
	ods->write2(av->getHP());
	ods->write2(av->getMaxHP());
	ods->write2(av->getMana());
	ods->write2(av->getMaxMana());
	ods->write2(av->getArmourClass());
	ods->write2(av->getTotalWeight());

	for (unsigned int i = 1; i <= 6; i++)
	{
		uint16 objid = av->getEquip(i);
		Item* item = getItem(objid);
		if (item) {
			ods->write4(item->getShape());
			ods->write4(item->getFrame());
		} else {
			ods->write4(0);
			ods->write4(0);
		}
	}
#endif
}
